#include "pwatch.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>


/*
 * Returns the content of a file
 */
int pwatch_get_content(const char *file, char *buf, const size_t buf_len)
{
	int fd;
	ssize_t n;

	fd = open(file, O_RDONLY);
	if (fd == -1)
		return -1;

	n = read(fd, buf, buf_len - 1);
	if (n < 0) {
		close(fd);
		return -1;
	}

	buf[buf_len - 1] = '\0';

	close(fd);

	return n;
}

/*
 * Returns the content of a file but will replace '\0' with ' '
 * It is useful from /proc/<pid>/cmdline
 */
int pwatch_get_content_text(const char *file, char *buf, const size_t buf_len)
{
	int err, i;

	err = pwatch_get_content(file, buf, buf_len);
	if (err < 0)
		return err;

	for (i = 0; i < err - 1; i++)
		if (buf[i] == '\0')
			buf[i] = ' ';

	return err;
}

/*
 * Show nice speed
 */
void pwatch_nice_output(char *buf, const size_t buf_len,
	const unsigned long long value)
{
	unsigned long long new;

	new = value;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu B", new);
		return;
	}

	new = new / 1024;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu K", new);
		return;
	}

	new = new / 1024;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu M", new);
		return;
	}

	new = new / 1024;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu G", new);
		return;
	}

	new = new / 1024;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu T", new);
		return;
	}

	new = new / 1024;
	if (new <= 9999) {
		snprintf(buf, buf_len, "%llu P", new);
		return;
	}

	snprintf(buf, buf_len, "  BIG!");
}

/*
 * Nice output of flags
 */
void pwatch_flags(char *buf, const size_t buf_len,
	const int f)
{
	char ibuf[14];

	snprintf(ibuf, sizeof(ibuf), "-------------");

	if (f & O_RDWR) {
		ibuf[0] = 'R';
		ibuf[1] = 'W';
	}

	if (f & O_RDONLY)
		ibuf[0] = 'R';

	if (f & O_WRONLY)
		ibuf[1] = 'W';

	if (f & O_APPEND)
		ibuf[2] = '+';

	if (f & O_ASYNC)
		ibuf[3] = 'A';

	if (f & O_CREAT)
		ibuf[4] = 'C';

	if (f & O_DIRECT)
		ibuf[5] = 'D';

	if (f & O_EXCL)
		ibuf[6] = 'X';

	if (f & O_LARGEFILE)
		ibuf[7] = 'L';

	if (f & O_NOATIME)
		ibuf[8] = 'T';

	if (f & O_NOCTTY)
		ibuf[9] = 'Y';

	if (f & O_NONBLOCK)
		ibuf[10] = 'K';

	if (f & O_SYNC)
		ibuf[11] = 'S';

	if (f & O_TRUNC)
		ibuf[12] = '0';

	snprintf(buf, buf_len, "%s", ibuf);
}

